/*
Copyright 2020 Appboxo pte. ltd.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package com.appboxo.appboxo_sdk

import android.annotation.SuppressLint
import android.app.Activity
import android.content.Context
import androidx.annotation.NonNull
import com.appboxo.js.params.CustomEvent
import com.appboxo.js.params.PaymentData
import com.appboxo.sdk.Appboxo
import com.appboxo.sdk.Config
import com.appboxo.sdk.Miniapp
import com.appboxo.sdk.MiniappConfig
import com.appboxo.ui.main.AppboxoActivity
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry
import java.lang.ref.WeakReference

class AppboxoSdkPlugin : FlutterPlugin, ActivityAware, MethodCallHandler {

    private val lifecycleListener = object : Miniapp.LifecycleListener {
        override fun onClose(miniapp: Miniapp) {
            lifecycleEventsStreamHandler.events?.success(
                mapOf(
                    "app_id" to miniapp.appId,
                    "lifecycle" to "onClose"
                )
            )
        }

        override fun onError(miniapp: Miniapp, message: String) {
            lifecycleEventsStreamHandler.events?.success(
                mapOf(
                    "app_id" to miniapp.appId,
                    "lifecycle" to "onError",
                    "error" to message
                )
            )
        }

        override fun onLaunch(miniapp: Miniapp) {
            lifecycleEventsStreamHandler.events?.success(
                mapOf(
                    "app_id" to miniapp.appId,
                    "lifecycle" to "onLaunch"
                )
            )
        }

        override fun onPause(miniapp: Miniapp) {
            lifecycleEventsStreamHandler.events?.success(
                mapOf(
                    "app_id" to miniapp.appId,
                    "lifecycle" to "onPause"
                )
            )
        }

        override fun onResume(miniapp: Miniapp) {
            lifecycleEventsStreamHandler.events?.success(
                mapOf(
                    "app_id" to miniapp.appId,
                    "lifecycle" to "onResume"
                )
            )
        }

        override fun onUserInteraction(miniapp: Miniapp) {
            lifecycleEventsStreamHandler.events?.success(
                mapOf(
                    "app_id" to miniapp.appId,
                    "lifecycle" to "onUserInteraction"
                )
            )
        }
    }
    private val customEventListener = object : Miniapp.CustomEventListener {
        override fun handle(
            miniAppActivity: AppboxoActivity,
            miniapp: Miniapp,
            customEvent: CustomEvent
        ) {
            customEventsStreamHandler.events?.success(
                mapOf(
                    "app_id" to miniapp.appId,
                    "request_id" to customEvent.requestId,
                    "type" to customEvent.type,
                    "error_type" to customEvent.errorType,
                    "payload" to customEvent.payload
                )
            )
        }
    }

    private val paymentEventListener = object : Miniapp.PaymentEventListener {
        override fun handle(
            miniAppActivity: AppboxoActivity,
            miniapp: Miniapp,
            paymentData: PaymentData
        ) {
            paymentEventsStreamHandler.events?.success(
                mapOf(
                    "app_id" to miniapp.appId,
                    "transaction_token" to paymentData.transactionToken,
                    "miniapp_order_id" to paymentData.miniappOrderId,
                    "amount" to paymentData.amount,
                    "currency" to paymentData.currency,
                    "status" to paymentData.status,
                    "hostapp_order_id" to paymentData.hostappOrderId,
                    "extra_params" to paymentData.extraParams
                )
            )
        }
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            "setConfig" -> {
                val clientId = call.argument<String>("client_id")!!
                val sandboxMode = call.argument<Boolean>("sandbox_mode") ?: false
                val multitaskMode = call.argument<Boolean>("multitask_mode") ?: true
                val theme = when (call.argument<String>("theme")) {
                    "light" -> Config.Theme.LIGHT
                    "dark" -> Config.Theme.DARK
                    else -> Config.Theme.SYSTEM
                }
                val isDebug = call.argument<Boolean>("is_debug") ?: false
                val showPermissionsPage = call.argument<Boolean>("show_permissions_page") ?: true
                val showClearCache = call.argument<Boolean>("show_clear_cache") ?: true

                Appboxo.setConfig(
                    Config.Builder()
                        .setClientId(clientId)
                        .sandboxMode(sandboxMode)
                        .multitaskMode(multitaskMode)
                        .setTheme(theme)
                        .permissionsPage(showPermissionsPage)
                        .showClearCache(showClearCache)
                        .debug(isDebug)
                        .build()
                )
            }

            "openMiniapp" -> {
                val appId = call.argument<String>("app_id")!!
                val data = call.argument<Map<String, Any>>("data")
                val theme = when (call.argument<String>("theme")) {
                    "light" -> Config.Theme.LIGHT
                    "dark" -> Config.Theme.DARK
                    "system" -> Config.Theme.SYSTEM
                    else -> null
                }
                val extraUrlParams = call.argument<Map<String, String>>("extra_url_params")
                val urlSuffix = call.argument<String>("url_suffix")
                val colors = call.argument<Map<String, String>>("colors")

                activity.get()?.also { context ->
                    Appboxo.getMiniapp(appId)
                        .setData(data)
                        .setCustomEventListener(customEventListener)
                        .setLifecycleListener(lifecycleListener)
                        .setPaymentEventListener(paymentEventListener)
                        .setAuthListener { _, miniapp ->
                            lifecycleEventsStreamHandler.events?.success(
                                mapOf(
                                    "app_id" to miniapp.appId,
                                    "lifecycle" to "onAuth"
                                )
                            )
                        }
                        .apply {
                            setConfig(
                                MiniappConfig.Builder().apply {
                                    theme?.also { setTheme(it) }
                                    extraUrlParams?.also { setExtraUrlParams(it) }
                                    urlSuffix?.also { setUrlSuffix(it) }
                                    colors?.also {
                                        setColors(
                                            it["primary_color"] ?: "",
                                            it["secondary_color"] ?: "",
                                            it["tertiary_color"] ?: ""
                                        )
                                    }
                                }.build()
                            )
                        }
                        .open(context)
                }
            }

            "closeMiniapp" -> {
                val appId = call.argument<String>("app_id")!!
                Appboxo.getExistingMiniapp(appId)?.close()
            }

            "setAuthCode" -> {
                val appId = call.argument<String>("app_id")!!
                val authCode = call.argument<String>("auth_code")!!
                Appboxo.getMiniapp(appId)
                    .setAuthCode(authCode)
            }

            "hideAllMiniapps" -> {
                Appboxo.hideMiniapps()
            }

            "sendPaymentEvent" -> {
                try {
                    val map = call.argument<Map<String, *>>("payment_event")!!
                    val appId = map["app_id"] as String
                    @Suppress("UNCHECKED_CAST") val paymentData = PaymentData(
                        transactionToken = map["transaction_token"] as String,
                        miniappOrderId = map["miniapp_order_id"] as String,
                        amount = map["amount"] as Double,
                        currency = map["currency"] as String,
                        status = map["status"] as String,
                        hostappOrderId = map["hostapp_order_id"] as String,
                        extraParams = map["extra_params"] as? Map<String, Any>
                    )
                    Appboxo.getMiniapp(appId).sendPaymentResult(paymentData)
                } catch (e: Exception) {
                    print(e)
                }
            }

            "sendEvent" -> {
                try {
                    val map = call.argument<Map<String, *>>("custom_event")!!
                    val appId = map["app_id"] as String

                    @Suppress("UNCHECKED_CAST")
                    val customEvent = CustomEvent(
                        requestId = map["request_id"] as? Int,
                        type = map["type"] as String,
                        errorType = map["error_type"] as? String,
                        payload = map["payload"] as Map<String, Any>
                    )
                    Appboxo.getMiniapp(appId).sendEvent(customEvent)
                } catch (e: Exception) {
                    print(e)
                }
            }

            "logout" -> {
                Appboxo.logout()
            }
        }
    }

    override fun onAttachedToEngine(binding: FlutterPlugin.FlutterPluginBinding) {
        initChannels(this, binding.binaryMessenger)
    }

    override fun onDetachedFromEngine(binding: FlutterPlugin.FlutterPluginBinding) {
        teardownChannels()
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        initAppboxoSdk(binding.activity)
    }

    override fun onDetachedFromActivityForConfigChanges() {
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    }

    override fun onDetachedFromActivity() {
    }

    companion object {
        @SuppressLint("StaticFieldLeak")
        private var activity: WeakReference<Activity?> = WeakReference(null)
        private val customEventsStreamHandler = EventsStreamHandler()
        private val paymentEventsStreamHandler = EventsStreamHandler()
        private val lifecycleEventsStreamHandler = EventsStreamHandler()
        private var methodChannel: MethodChannel? = null
        private var customEventChannel: EventChannel? = null
        private var paymentEventChannel: EventChannel? = null
        private var lifecycleEventChannel: EventChannel? = null

        @JvmStatic
        fun registerWith(registrar: PluginRegistry.Registrar) {
            initAppboxoSdk(registrar.activity()!!)
            initChannels(AppboxoSdkPlugin(), registrar.messenger())
        }

        private fun initAppboxoSdk(activity: Activity) {
            this.activity = WeakReference(activity)
            Appboxo.init(activity.application)
        }

        private fun initChannels(plugin: AppboxoSdkPlugin, binaryMessenger: BinaryMessenger) {
            methodChannel = MethodChannel(binaryMessenger, "appboxo_sdk")
            methodChannel?.setMethodCallHandler(plugin)

            customEventChannel = EventChannel(binaryMessenger, "appboxo_sdk_custom_events")
            customEventChannel?.setStreamHandler(customEventsStreamHandler)

            paymentEventChannel = EventChannel(binaryMessenger, "appboxo_sdk_payment_events")
            paymentEventChannel?.setStreamHandler(paymentEventsStreamHandler)

            lifecycleEventChannel = EventChannel(binaryMessenger, "appboxo_sdk_lifecycle")
            lifecycleEventChannel?.setStreamHandler(lifecycleEventsStreamHandler)
        }

        private fun teardownChannels() {
            methodChannel?.setMethodCallHandler(null)
            customEventChannel?.setStreamHandler(null)
            paymentEventChannel?.setStreamHandler(null)
            lifecycleEventChannel?.setStreamHandler(null)
            methodChannel = null
            customEventChannel = null
            paymentEventChannel = null
            lifecycleEventChannel = null
        }
    }
}
